% Haizhou scordablature input, Tripod output.

\include "tripod-data.ly"

tuning = #60

% Override a function that tripod-setup.ly uses
#(define (set-tripod-names semitoeSize sharpSize inchSize commaSize)
        (ly:set-default-scale (ly:make-scale
            (tripodNominals (+ semitoeSize sharpSize) inchSize))))

% (toe . semitoes)
tuningTable = #(list->vector (map cons
  '(
    1 1 1 1
    2 2 3 3 3 4 4 5 5 6 6 6
    7 7 8 8 8 9 9 9
  )
  '(
    -2 -1 0 1
    -1 0 -1 0 1 -1 0 -1 0 -1 0 1
    -1 0 -1 0 1 -1 0 1
  )))
mappingSize = 24
mappingStart = #-4 % relative to middle C

% The magic function that converts to a tripod pitch
% from a scordablature pitch.
% Transposes from the treble clef to the register specified.
#(define (tripod-pitch register p)
  (let* ((hsteps (- (ly:pitch-semitones p) mappingStart))
         (reg (floor (/ hsteps mappingSize)))
         (hai-degree (- hsteps (* reg mappingSize)))
         (trip-pitch (vector-ref tuningTable hai-degree))
         (toe (- (car trip-pitch) 1))
         (alteration (* (cdr trip-pitch) (best-semitoe tuning))))
      (ly:make-pitch (+ register reg -4) toe alteration)))

% Convert all notes.
% This is copied from "Transposing music with minimum accidentals"
% In the "Pitches" snippets in the documentation.
#(define (haizhou-to-tripod register music)
  (let* ((es (ly:music-property music 'elements))
         (e (ly:music-property music 'element))
         (p (ly:music-property music 'pitch)))
    (if (pair? es)
       (ly:music-set-property!
         music 'elements
         (map (lambda (x) (haizhou-to-tripod register x)) es)))
    (if (ly:music? e)
       (ly:music-set-property!
         music 'element
         (haizhou-to-tripod register e)))
    (if (ly:pitch? p)
       (begin
         (set! p (tripod-pitch register p))
         (ly:music-set-property! music 'pitch p)))
    music))

% Convert to Tripod pitches in a given register.
haizhou = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (haizhou-to-tripod register m))


% For standard Lilypond accidentals
tripodGlyphs = \fifthGlyphs

tripodStrings = #(make-magic-strings tuning)
didymusStrings = #(make-didymus-strings tuning)

\include "tripod-setup.ly"
