from __future__ import division
"""Script to write Scala files for Tripod/Pengcheng mapping"""

pengcheng_toes = (
  1, 2, 2, 3, 3,
  4, 4, 5, 5, 6, 6, 7,
  7, 8, 8, 9, 9,
  11, 11, 12, 12, 13, 13, 14,
  14, 15, 15, 16, 16,
  17, 17, 18, 18, 19, 19, 19,
  21
)

pengcheng_shifts = (
  0, -1, 0, -1, 0,
  -1, 0, -1, 0, -1, 0, -1,
  0, -1, 0, -1, 0,
  -1, 0, -1, 0, -1, 0, -1,
  0, -1, 0, -1, 0,
  -1, 0, -1, 0, -1, 0, 1,
  0
)

pengcheng_header = """! %%s
!
Tripod-based Pengcheng mapping for magic temperament.
! Tuned to %s.
! This scale covers two octaves with an average of 18 notes per octave,
! including a full 19 note MOS.
 %%i
!
"""

haizhou_toes = (
2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 6,
7, 7, 8, 8, 8, 9, 9, 9, 11, 11, 11, 11,
12
)

haizhou_shifts = (
-1, 0, -1, 0, 1, -1, 0, -1, 0, -1, 0, 1,
-1, 0, -1, 0, 1, -1, 0, 1, -2, -1, 0, 1,
-1
)

haizhou_header = """! %%s
!
Tripod-based Haizhou mapping for magic temperament.
! Tuned to %s.
! This embeds a 22 note MOS into 24 notes.
! 9 out of 10 black notes are the tripod scale.
 %%i
!
"""

def tripod_to_cents(tripod_degree, shift, semitoe_size, inch_size):
    octave_size = semitoe_size*19 + inch_size*3
    octaves = tripod_degree // 10
    toes = (tripod_degree%10) - 1
    semitoes = 2*toes + shift
    inches = toes//3
    pitch = octaves*octave_size + semitoes*semitoe_size + inches*inch_size
    return pitch*12e2

def write_scl_file(sdesc, header, toes, shifts, semitoe_size, inch_size):
    assert len(toes) == len(shifts)
    filename = "%s.scl" % sdesc
    output = open(filename, "w")
    output.write(header%(filename, len(shifts)-1))
    first = tripod_to_cents(toes[0], shifts[0], semitoe_size, inch_size)
    for t, s in zip(toes, shifts)[1:]:
        output.write("%8.3f\n" %
                (tripod_to_cents(t, s, semitoe_size, inch_size) - first))
    output.close()

write_scl_file("pengcheng_60", pengcheng_header%"60 note equal temperament",
        pengcheng_toes, pengcheng_shifts, 3/60, 1/60)
write_scl_file("pengcheng_41", pengcheng_header%"41 note equal temperament",
        pengcheng_toes, pengcheng_shifts, 2/41, 1/41)
write_scl_file("pengcheng_rms", pengcheng_header%"TOP-RMS tuning",
        pengcheng_toes, pengcheng_shifts,
        0.049164225335868172, 0.02226057548733773)

write_scl_file("haizhou_60", haizhou_header%"60 note equal temperament",
        haizhou_toes, haizhou_shifts, 3/60, 1/60)
write_scl_file("haizhou_41", haizhou_header%"41 note equal temperament",
        haizhou_toes, haizhou_shifts, 2/41, 1/41)
write_scl_file("haizhou_rms", haizhou_header%"TOP-RMS tuning",
        haizhou_toes, haizhou_shifts,
        0.049164225335868172, 0.02226057548733773)

