% Tripod input with Haizhou scordablature output.
% No planar cleverness, and inches in the score are not supported.

\include "tripod-data.ly"

% Treat the tripod input as 60-equal
% although it isn't output to MIDI
semitoeSize = #3/10
inchSize = #1/10

#(set-tripod-names semitoeSize semitoeSize inchSize inchSize)

% now reset the output scale
majorScale = #'#(0 1 2 5/2 7/2 9/2 11/2)
#(ly:set-default-scale (ly:make-scale majorScale))

% Only pitches in this hash are recognized
% ((tripod-degree . alteration) . (white-note . alteration))
tripodToHaizhou = #'(
    ((1 . -2) . (-2 . -1)) ; Ab
    ((1 . -1) . (-2 . 0)) ; A
    ((1 . 0) . (-1 . -1)) ; Bb
    ((1 . 1) . (-1 . 0)) ; B
    ((2 . -1) . (0 . 0)) ; C
    ((2 . 0) . (0 . 1)) ; C#
    ((2 . 1) . (1 . 0)) ; D
    ((3 . -1) . (1 . 0))
    ((3 . 0) . (2 . -1)) ; Eb
    ((3 . 1) . (2 . 0)) ; E
    ((4 . -1) . (3 . 0)) ; F
    ((4 . 0) . (3 . 1)) ; F#
    ((4 . 1) . (4 . 0)) ; G
    ((5 . -1) . (4 . 0))
    ((5 . 0) . (4 . 1)) ; G#
    ((5 . 1) . (5 . 0)) ; A
    ((6 . -1) . (5 . 0))
    ((6 . 0) . (6 . -1)) ; Bb
    ((6 . 1) . (6 . 0)) ; B
    ((7 . -1) . (7 . 0)) ; C
    ((7 . 0) . (7 . 1)) ; C#
    ((7 . 1) . (8 . 0)) ; D
    ((8 . -1) . (8 . 0))
    ((8 . 0) . (9 . -1)) ; Eb
    ((8 . 1) . (9 . 0)) ; E
    ((9 . -1) . (10 . 0)) ; F
    ((9 . 0) . (10 . 1)) ; F#
    ((9 . 1) . (11 . 0)) ; G
    ((9 . 2) . (11 . 1)) ; G#
)

% The magic function that converts a tripod pitch
% to a scordablature pitch.
% Transposes from register 3 to that specified.
% Fails if the conversion isn't defined.
#(define (haizhou-pitch register p)
  (let* ((reg (+ (ly:pitch-octave p) register -3))
         (semitoes (/ (ly:pitch-alteration p) semitoeSize))
         (tripod-degree (+ (ly:pitch-notename p) 1))
         (hai-reg (* reg 2))
         (hai-pitch (cdr
            (assoc (cons tripod-degree semitoes) tripodToHaizhou)))
         (hai-degree (car hai-pitch))
         (alteration (/ (cdr hai-pitch) 2)))
      (ly:make-pitch hai-reg hai-degree alteration)))

% Convert all notes.
% This is copied from "Transposing music with minimum accidentals"
% In the "Pitches" snippets in the documentation.
#(define (tripod-to-haizhou register music)
  (let* ((es (ly:music-property music 'elements))
         (e (ly:music-property music 'element))
         (p (ly:music-property music 'pitch)))
    (if (pair? es)
       (ly:music-set-property!
         music 'elements
         (map (lambda (x) (tripod-to-haizhou register x)) es)))
    (if (ly:music? e)
       (ly:music-set-property!
         music 'element
         (tripod-to-haizhou register e)))
    (if (ly:pitch? p)
       (begin
         (set! p (haizhou-pitch register p))
         (ly:music-set-property! music 'pitch p)))
    music))

% Convert to Haizhou pitches,
% and correct for a previous transposition to register
haizhou = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (tripod-to-haizhou (- 7 register) m))

% Tripod note names in a given register, ignored here.
% Also does the haizhou conversion.
tripod = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (tripod-to-haizhou 4 m))

\paper{
    newStaffSize = #22
    #(define fonts (make-pango-font-tree "URW Bookman L" "Sans" "Mono"
                    (/ newStaffSize 20)))
}
lyricsize=\tiny

% All clefs are played as treble,
% assuming the music in them is appropriately transposed.
% I can't work out how to automatically indicate
% that the pitch output should be different.
fourfiveclef = {
  \clef treble
}
fourclef = {
  \clef treble
}
threeclef = {
  \clef treble
}
