% Tripod input with output for the Haizhou keyboard mapping.
% MIDI in 60-equal.
% No planar cleverness, and inches in the score are not supported.

\include "tripod-data.ly"

semitoeSize = #(best-semitoe 60)
inchSize = #(best-inch 60)

% The Haizhou mapping starts on the G#/Ab key
% The LilyPond scale starts on the A key
haizhouToes = #'#(
1 1 1 1
2 2 3 3 3 4 4 5 5 6 6 6
7 7 8 8 8 9 9 9
)

haizhouShifts = #'#(
-2 -1 0 1
-1 0 -1 0 1 -1 0 -1 0 -1 0 1
-1 0 -1 0 1 -1 0 1
)

% Only pitches in this hash are recognized
% ((tripod-degree . alteration) . (white-note . alteration))
tripodToHaizhou = #'(
    ((1 . -2) . (0 . -1)) ; Ab
    ((1 . -1) . (0 . 0)) ; A
    ((1 . 0) . (1 . -1)) ; Bb
    ((1 . 1) . (1 . 0)) ; B
    ((2 . -1) . (2 . 0)) ; C
    ((2 . 0) . (2 . 1)) ; C#
    ((2 . 1) . (3 . 0)) ; D
    ((3 . -1) . (3 . 0))
    ((3 . 0) . (4 . -1)) ; Eb
    ((3 . 1) . (4 . 0)) ; E
    ((4 . -1) . (5 . 0)) ; F
    ((4 . 0) . (5 . 1)) ; F#
    ((4 . 1) . (6 . 0)) ; G
    ((5 . -1) . (6 . 0))
    ((5 . 0) . (6 . 1)) ; G#
    ((5 . 1) . (7 . 0)) ; A
    ((6 . -1) . (7 . 0))
    ((6 . 0) . (8 . -1)) ; Bb
    ((6 . 1) . (8 . 0)) ; B
    ((7 . -1) . (9 . 0)) ; C
    ((7 . 0) . (9 . 1)) ; C#
    ((7 . 1) . (10 . 0)) ; D
    ((8 . -1) . (10 . 0))
    ((8 . 0) . (11 . -1)) ; Eb
    ((8 . 1) . (11 . 0)) ; E
    ((9 . -1) . (12 . 0)) ; F
    ((9 . 0) . (12 . 1)) ; F#
    ((9 . 1) . (13 . 0)) ; G
    ((9 . 2) . (13 . 1)) ; G#
)

% Make the first written note pitch 0.
#(define (haizhou-pitch note) ( +
    (* 2 semitoeSize (- (vector-ref haizhouToes note) 1))
    (* semitoeSize (+ (vector-ref haizhouShifts note) 1))
    (* inchSize (truncate (/ (- (vector-ref haizhouToes note) 1) 3)))
  ))

% 14 white notes, 24 keys per octave
whiteNotes = #'(1 3 4 6 8 9 11 13 15 16 18 20 21 23)

#(ly:set-default-scale (ly:make-scale
    (list->vector (map haizhou-pitch whiteNotes))))

#(define (tripod-pitch degree semitoes)
    (let ((lookup (assoc (cons degree semitoes) tripodToHaizhou)))
        (if (eq? lookup #f) #f
          (ly:make-pitch -1 (cadr lookup) (* semitoeSize (cddr lookup))))
          ))

haizhouPitchNames = #(map
    (lambda (name) (cons (car name)
        (let ((shifts (cddr name)))
            (tripod-pitch (cadr name) (+ (car shifts) (cadr shifts))))))
        planarNames)


#(ly:parser-set-note-names parser haizhouPitchNames)

% The standard glyphs but with different sizes
fetaGlyphs = #'(
    (3/5 . "accidentals.doublesharp")
    (3/10 . "accidentals.sharp")
    (0 . "accidentals.natural")
    (-3/10 . "accidentals.flat")
    (-3/5 . "accidentals.flatflat")
    )

\paper{
    newStaffSize = #22
    #(define fonts (make-pango-font-tree "URW Bookman L" "Sans" "Mono"
                    (/ newStaffSize 20)))
}
lyricsize=\tiny

\layout {
  \context {
    \Score
    \override Accidental #'glyph-name-alist = \fetaGlyphs
    \override KeySignature #'glyph-name-alist = \fetaGlyphs
    \override AccidentalCautionary #'glyph-name-alist = \fetaGlyphs
    \override TrillPitchAccidental #'glyph-name-alist = \fetaGlyphs
    \override AmbitusAccidental #'glyph-name-alist = \fetaGlyphs
  }
}

% Tripod note names in a given register.
tripod = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (octavate register m))

fourfiveclef = {
  \clef treble
  \set Staff.middleCPosition = #-8
  \transposition hoj % tan=C, near enough
}
fourclef = {
  \clef treble
  \set Staff.middleCPosition = #-8
  \transposition hoj % tan=C, near enough
}
threeclef = {
  \clef bass
  \set Staff.middleCPosition = #4
  \transposition hoj % tan=C, near enough
}
