\include "sagittal.ly"

#(define (best-semitoe ET) (- 6 (* 3 (best-pace ET))))
#(define (best-inch ET) (- (best-pace ET) (* 3 (best-toe ET))))
#(define (best-sharp ET) (- (best-toe ET) (best-semitoe ET)))
#(define (best-comma ET)
    (let ((tone (+ (best-toe ET) (best-semitoe ET) (best-inch ET)))) ; 9:8
        (- (* 2 tone) (best-pace ET))))

#(define (best-pace ET)
    ; nearest step to 5:4
    (let ((pace (nint (* ET 0.32192809488736235))))
        (* (/ pace ET) 6)))

#(define (best-toe ET)
    ; nearest step to 8:7
    (let ((toe
        (let ((twotoe (nint (* ET 0.19264507794239583))))
            (if (= (modulo twotoe 2) 0)
                (/ twotoe 2)
                ; 8:7 not an even number of steps,
                ; so use 16:15 instead
                (let ((pace (nint (* ET 0.32192809488736235))) ; nearest to 5:4
                    (stride (nint (* ET 0.4150374992788437)))) ; nearest to 4:3
                    (- stride pace))))))
        (* (/ toe ET) 6)))

#(define (nint x) (inexact->exact (round x)))

% name flat-prefix sharp-prefix
degreeNames = #'(
    (yan ya ya) (tan ta ta) (eth e e)
    (meth me me) (pip pu pi) (seth se se)
    (leth le le) (hov ho ho) (dov do do))

% flatten sharpen semitoes sharps inches commas
tripodAlterations = #'(
    (b j 1 0 0 0)
    (bub jij 2 0 0 0)
    (d u 0 0 1 0)
    (bid ju 1 0 1 0) ; inch
    (cod cu 0 0 0 1) ; didymus
    (f s 0 1 0 0)
    (bud jiu 1 1 0 0)
)

#(define (symbol-append a b)
    (if b
        (string->symbol (string-append
            (symbol->string a) (symbol->string b)))
        a))

#(define (apply-alterations names degree)
    (let next-alteration ((alterations tripodAlterations))
        (if (null? alterations) (list (list (car names) degree 0 0 0 0))
            (let* ((alteration (car alterations))
                   (shifts (cddr alteration)))
                (cons (cons (symbol-append (cadr names) (car alteration))
                        (cons degree (map - shifts)))
                (cons (cons (symbol-append (caddr names) (cadr alteration))
                        (cons degree shifts))
                    (next-alteration (cdr alterations))))))))

tripodNames = #(let next-degree ((nominals degreeNames) (degree 1))
        (if (null? nominals) '()
            (append (apply-alterations (car nominals) degree)
                    (next-degree (cdr nominals) (+ 1 degree)))))

% suffixes that mean nothing here, but may disambiguate trojan pitches
#(define (apply-warts names)
    (if (null? names) '()
        (let ((name (car names)))
            (append (list
                name
                (cons (symbol-append (car name) 'mf) (cdr name))
                (cons (symbol-append (car name) 'ms) (cdr name)))
            (apply-warts (cdr names))))))

planarNames = #(apply-warts tripodNames)

#(define (make-strings lookup)
    (map (lambda (entry)
        (list (car entry) (cadr entry)
            (sagittalString (cddr entry))
            (sagittalXExtent (cddr entry))
            (sagittalYExtent (cddr entry))
            ))
        lookup))

% For the trojans
#(define (ratio-strings lookup)
    (map (lambda (entry)
        (list (car entry)
            (sagittalString (cdr entry))
            (sagittalXExtent (cdr entry))
            (sagittalYExtent (cdr entry))))
        lookup))

% as (semitoes inches string X-extent Y-extent) once processed
magicStrings = #(make-strings '(
    ( 2  0  rightscrolltripleup) ; 15:14
    ( 1  1  doubleleftbarbdoubleup)
    ( 1  0  leftbarbdoubleup) ;3/60
    ( 0  1  rightarcup) ; 2/72
    ( 0  0  natural)
    ( 0 -1  rightarcdown) ; 2/72
    (-1  0  leftbarbdoubledown) ; 3/60
    (-1 -1  doubleleftbarbdoubledown)
    (-2 0   rightscrolltripledown) ; 14:15
))

% as (sharps commas symbol)
twinStrings = #(make-strings '(
    ( 1  0   rightarcdoubleup)
    ( 0  1   leftbarbup) ; 81:80
    ( 0 -1   leftbarbdown) ;81:80
    (-1  0   rightarcdoubledown)
))

% like magicStrings but with syntonic comma shifts
inchStrings = #(make-strings '(
    ( 2  0  rightscrolltripleup) ; 15:14
    ( 1  1  doubleleftbarbdoubleup)
    ( 1  0  leftbarbdoubleup) ; 3/60
    ( 0  1  leftbarbup) ; 25:24
    ( 0  0  natural)
    ( 0 -1  leftbarbdown) ; 25:24
    (-1  0  leftbarbdoubledown) ; 3/60
    (-1 -1  doubleleftbarbdoubledown)
    (-2  0  rightscrolltripledown) ; 14:15
))

#(define (tripodString entry semitoeSize inchSize)
    (cons (+ (* semitoeSize (car entry))
             (* inchSize (cadr entry)))
          (caddr entry)))

#(define (xExtent entry semitoeSize inchSize)
    (cons (+ (* semitoeSize (car entry))
             (* inchSize (cadr entry)))
          (cadddr entry)))

#(define (yExtent entry semitoeSize inchSize)
    (cons (+ (* semitoeSize (car entry))
             (* inchSize (cadr entry)))
          (car (cddddr entry))))

% The thing the default scale is set to :
#(define (tripodNominals toeSize inchSize)
    (list->vector
        (map (lambda (pitch)
                (+ (* toeSize (car pitch)) (* inchSize (cdr pitch))))
             '((0 . 0) (1 . 0) (2 . 0)
               (3 . 1) (4 . 1) (5 . 1)
               (6 . 2) (7 . 2) (8 . 2)
             ))))

% Place 9 note scale in 10 staff positions
#(define (tripod-position pitch)
    (+ (* 10 (ly:pitch-octave pitch))
       (ly:pitch-notename pitch)))

#(define (make-planar-strings ET) (append
    (map (lambda (entry) (tripodString entry
        (best-semitoe ET) (best-inch ET))) magicStrings)
    (map (lambda (entry) (tripodString entry
        (best-sharp ET) (best-comma ET))) twinStrings)
))

#(define (make-magic-strings ET)(map (lambda (entry)
    (tripodString entry
        (best-semitoe ET) (best-inch ET))) magicStrings))

#(define (make-didymus-strings ET)(map (lambda (entry)
    (tripodString entry
        (best-semitoe ET) (best-inch ET))) inchStrings))

#(define (make-tripod-names semitoeSize sharpSize inchSize commaSize)
    (map (lambda (name)
        (let ((degree (cadr name))
              (shifts (cddr name)))
            (cons (car name)
                (ly:make-pitch -1 (- degree 1)
                    (+ (* semitoeSize (car shifts))
                       (* sharpSize (cadr shifts))
                       (* inchSize (caddr shifts))
                       (* commaSize (cadddr shifts)))))))
        planarNames))

#(define (set-tripod-names semitoeSize sharpSize inchSize commaSize)
    (begin
        (ly:set-default-scale (ly:make-scale
            (tripodNominals (+ semitoeSize sharpSize) inchSize)))
        (ly:parser-set-note-names parser (make-tripod-names
            semitoeSize sharpSize inchSize commaSize))))

tripodClefGlyphs = #'#(
    "zero" "one" "two" "three" "four" "five" "six" "seven" "eight" "nine")

tripodClef = #(define-music-function (parser location octave) (integer?)
#{
    % Sagittal glyphs:
    \override Accidental #'stencil = #ly:text-interface::print
    \override Accidental #'font-name = #"Sagittal"
    \override Accidental #'font-size = #4
    \override Accidental #'text = #(lambda (grob)
        (cdr (assoc (ly:grob-property grob 'alteration) tripodStrings)))
    \override Accidental #'X-extent = #(lambda (grob)
        (cdr (assoc (ly:grob-property grob 'alteration) xExtents)))
    \override Accidental #'Y-extent = #(lambda (grob)
        (cdr (assoc (ly:grob-property grob 'alteration) yExtents)))

    % Feta glyphs (reduntant?)
    \override Accidental #'glyph-name-alist = \tripodGlyphs
    \override KeySignature #'glyph-name-alist = \tripodGlyphs
    \override AccidentalCautionary #'glyph-name-alist = \tripodGlyphs
    \override TrillPitchAccidental #'glyph-name-alist = \tripodGlyphs
    \override AmbitusAccidental #'glyph-name-alist = \tripodGlyphs

    % Staff setup:
    \override Staff.StaffSymbol #'line-positions = #'(-3 0 3)
    \override Staff.Clef #'font-size = #+6
    \set Staff.clefGlyph = #(vector-ref tripodClefGlyphs $octave)
    \set Staff.middleCPosition = #(- 36 (* 10 $octave))
    \set Staff.clefPosition = #-4
    \set Staff.extraNatural = ##f
    % set tan=C
    \set Staff.instrumentTransposition = #(ly:make-pitch 0 1 0)
#})

doubleTripodClef = #(define-music-function (parser location lowoctave) (integer?)
#{
    \tripodClef #$lowoctave
    \override Staff.StaffSymbol #'line-positions = #'(-3 0 3 7 10 3)
#})

% For standard Lilypond accidentals in 72-equal
sixthGlyphs = #'(
    (7/12 . "accidentals.sharp.arrowup")
    (1/2 . "accidentals.sharp")
    (5/12 . "accidentals.sharp.arrowdown")
    (1/3 . "accidentals.sharp.slashslashslash.stem")
    (1/4 . "accidentals.sharp.slashslash.stem")
    (1/6 . "accidentals.natural.arrowup")
    (1/12 . "accidentals.natural.arrowup")
    (0 . "accidentals.natural")
    (-1/12 . "accidentals.natural.arrowdown")
    (-1/6 . "accidentals.natural.arrowdown")
    (-1/4 . "accidentals.flat.slash")
    (-1/3 . "accidentals.mirroredflat")
    (-5/12 . "accidentals.flat.arrowup")
    (-1/2 . "accidentals.flat")
    (-7/12 . "accidentals.flat.arrowdown")
)

% For standard Lilypond accidentals in 60-equal
fifthGlyphs = #'(
    (3/5 . "accidentals.sharp.arrowup")
    (1/2 . "accidentals.sharp")
    (2/5 . "accidentals.sharp.arrowdown")
    (3/10 . "accidentals.sharp.slashslashslash.stem")
    (1/5 . "accidentals.sharp.slashslash.stem")
    (1/10 . "accidentals.natural.arrowup")
    (0 . "accidentals.natural")
    (-1/10 . "accidentals.natural.arrowdown")
    (-1/5 . "accidentals.mirroredflat.backslash")
    (-3/10 . "accidentals.flat.slash")
    (-2/5 . "accidentals.flat.arrowup")
    (-1/2 . "accidentals.flat")
    (-3/5 . "accidentals.flat.arrowdown"))

